#
# This is the SPEC file for creating binary RPMs for the Dom0.
#
#
# The Qubes OS Project, http://www.qubes-os.org
#
# Copyright (C) 2013  Marek Marczykowski  <marmarek@invisiblethingslab.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#

%{!?python_sitearch: %define python_sitearch %(%{__python} -c "from distutils.sysconfig import get_python_lib; print get_python_lib(1)")}

%define _dracutmoddir	/usr/lib/dracut/modules.d

Name:		qubes-core-dom0-linux
Version:	4.0.31
Release:	1%{?dist}
Summary:	Linux-specific files for Qubes dom0

Group:		Qubes
Vendor:		Invisible Things Lab
License:	GPL
URL:		http://www.qubes-os.org

BuildRequires:  ImageMagick
BuildRequires:  pandoc
BuildRequires:  qubes-utils-devel >= 3.1.3
BuildRequires:  qubes-libvchan-devel
BuildRequires:  gcc
Requires:	qubes-core-dom0
Requires:	python3-qubesadmin
Requires:	qubes-core-admin-client
Requires:	qubes-utils >= 3.1.3
Requires:	qubes-utils-libs >= 4.0.16
Requires:	qubes-rpm-oxide
Conflicts:	qubes-core-dom0 < 4.0.23
Requires:	%{name}-kernel-install
Requires:	xdotool
Requires:	createrepo_c
Requires:	rpm >= 4.14

Source0: %{name}-%{version}.tar.gz

%description
Linux customizations required to use system as Qubes dom0.
Additionally some graphical elements for every Linux desktop envirnment (icons,
appmenus etc).

%package kernel-install
Summary:	Kernel install hook for Xen-based system

# get rid of os-prober, it tries to mount and parse all the block devices in
# the system, including loop*
Provides: os-prober
Obsoletes: os-prober

%description kernel-install
Kernel install hook for Xen-based system.

%prep
%setup -q

%build
export BACKEND_VMM=xen
(cd dom0-updates; make)
(cd qrexec; make)
(cd file-copy-vm; make)
(cd doc; make manpages)

%install

## Appmenus
install -d $RPM_BUILD_ROOT/etc/qubes-rpc/policy
cp qubesappmenus/qubes.SyncAppMenus.policy $RPM_BUILD_ROOT/etc/qubes-rpc/policy/qubes.SyncAppMenus

### Dom0 updates
install -D dom0-updates/qubes-dom0-updates.cron $RPM_BUILD_ROOT/etc/cron.daily/qubes-dom0-updates.cron
install -D dom0-updates/qubes-dom0-update $RPM_BUILD_ROOT/usr/bin/qubes-dom0-update
install -D dom0-updates/qubes-receive-updates $RPM_BUILD_ROOT/usr/libexec/qubes/qubes-receive-updates
install -D dom0-updates/patch-dnf-yum-config $RPM_BUILD_ROOT/usr/lib/qubes/patch-dnf-yum-config
install -m 0644 -D dom0-updates/qubes-cached.repo $RPM_BUILD_ROOT/etc/yum.real.repos.d/qubes-cached.repo
install -D dom0-updates/qfile-dom0-unpacker $RPM_BUILD_ROOT/usr/libexec/qubes/qfile-dom0-unpacker
install -m 0644 -D dom0-updates/qubes.ReceiveUpdates $RPM_BUILD_ROOT/etc/qubes-rpc/qubes.ReceiveUpdates
install -m 0664 -D dom0-updates/qubes.ReceiveUpdates.policy $RPM_BUILD_ROOT/etc/qubes-rpc/policy/qubes.ReceiveUpdates

install -d $RPM_BUILD_ROOT/var/lib/qubes/updates

# Qrexec
mkdir -p $RPM_BUILD_ROOT/usr/bin $RPM_BUILD_ROOT/usr/sbin
install qrexec/qrexec-daemon $RPM_BUILD_ROOT/usr/sbin/
install qrexec/qrexec-client $RPM_BUILD_ROOT/usr/bin/
# XXX: Backward compatibility
ln -s ../../bin/qrexec-client $RPM_BUILD_ROOT/usr/lib/qubes/qrexec-client
ln -s ../../sbin/qrexec-daemon $RPM_BUILD_ROOT/usr/lib/qubes/qrexec-daemon
cp qrexec/qubes-rpc-multiplexer $RPM_BUILD_ROOT/usr/lib/qubes

### pm-utils
mkdir -p $RPM_BUILD_ROOT/usr/lib64/pm-utils/sleep.d
cp pm-utils/52qubes-pause-vms $RPM_BUILD_ROOT/usr/lib64/pm-utils/sleep.d/
mkdir -p $RPM_BUILD_ROOT/usr/lib/systemd/system
cp pm-utils/qubes-suspend.service $RPM_BUILD_ROOT/usr/lib/systemd/system/

### Dracut module
mkdir -p $RPM_BUILD_ROOT/etc/dracut.conf.d
cp dracut/dracut.conf.d/* $RPM_BUILD_ROOT/etc/dracut.conf.d/

mkdir -p $RPM_BUILD_ROOT%{_dracutmoddir}
cp -r dracut/modules.d/* $RPM_BUILD_ROOT%{_dracutmoddir}/

### Others
mkdir -p $RPM_BUILD_ROOT/etc/sysconfig
install -m 0644 -D system-config/limits-qubes.conf $RPM_BUILD_ROOT/etc/security/limits.d/99-qubes.conf
install -D system-config/cpufreq-xen.modules $RPM_BUILD_ROOT/etc/sysconfig/modules/cpufreq-xen.modules
install -m 0440 -D system-config/qubes.sudoers $RPM_BUILD_ROOT/etc/sudoers.d/qubes
install -D system-config/polkit-1-qubes-allow-all.rules $RPM_BUILD_ROOT/etc/polkit-1/rules.d/00-qubes-allow-all.rules
install -D system-config/qubes-dom0.modules $RPM_BUILD_ROOT/etc/sysconfig/modules/qubes-dom0.modules
install -D system-config/qubes-sync-clock.cron $RPM_BUILD_ROOT/etc/cron.d/qubes-sync-clock.cron
install -D system-config/lvm-cleanup.cron-daily $RPM_BUILD_ROOT/etc/cron.daily/lvm-cleanup
install -d $RPM_BUILD_ROOT/etc/udev/rules.d
install -m 644 system-config/00-qubes-ignore-devices.rules $RPM_BUILD_ROOT/etc/udev/rules.d/
install -m 644 system-config/12-qubes-ignore-lvm-devices.rules $RPM_BUILD_ROOT/etc/udev/rules.d/
install -m 644 -D system-config/disable-lesspipe.sh $RPM_BUILD_ROOT/etc/profile.d/zz-disable-lesspipe.sh
install -m 755 -D system-config/kernel-grub2.install $RPM_BUILD_ROOT/usr/lib/kernel/install.d/80-grub2.install
install -m 755 -D system-config/kernel-xen-efi.install $RPM_BUILD_ROOT/usr/lib/kernel/install.d/90-xen-efi.install
install -m 755 -D system-config/kernel-remove-bls.install $RPM_BUILD_ROOT/usr/lib/kernel/install.d/99-remove-bls.install
install -m 644 -D system-config/75-qubes-dom0.preset \
    $RPM_BUILD_ROOT/usr/lib/systemd/system-preset/75-qubes-dom0.preset
install -m 644 -D system-config/99-qubes-default-disable.preset \
    $RPM_BUILD_ROOT/usr/lib/systemd/system-preset/99-qubes-default-disable.preset
install -d $RPM_BUILD_ROOT/etc/dnf/protected.d
install -m 0644 system-config/dnf-protected-qubes-core-dom0.conf  \
        $RPM_BUILD_ROOT/etc/dnf/protected.d/qubes-core-dom0.conf


touch $RPM_BUILD_ROOT/var/lib/qubes/.qubes-exclude-block-devices

# file copy to VM
install -m 755 file-copy-vm/qfile-dom0-agent $RPM_BUILD_ROOT/usr/lib/qubes/
install -m 755 file-copy-vm/qvm-copy-to-vm $RPM_BUILD_ROOT/usr/bin/
install -m 755 file-copy-vm/qvm-copy $RPM_BUILD_ROOT/usr/bin/
ln -s qvm-copy-to-vm $RPM_BUILD_ROOT/usr/bin/qvm-move-to-vm
ln -s qvm-copy $RPM_BUILD_ROOT/usr/bin/qvm-move

### Documentation
(cd doc; make DESTDIR=$RPM_BUILD_ROOT install)

%pre
if ! grep -q ^qubes: /etc/group ; then
		groupadd qubes
fi

%post

/usr/lib/qubes/patch-dnf-yum-config

systemctl enable qubes-suspend.service >/dev/null 2>&1

# migrate dom0-updates check disable flag
if [ $1 -ge 2 ]; then
    if [ -e /var/lib/qubes/updates/disable-updates ]; then
        qvm-features dom0 service.qubes-update-check ''
        rm -f /var/lib/qubes/updates/disable-updates
    fi
fi

%preun
if [ "$1" = 0 ] ; then
	# no more packages left

    systemctl disable qubes-suspend.service > /dev/null 2>&1
fi

%triggerin -- PackageKit
# dom0 have no network, but still can receive updates (qubes-dom0-update)
sed -i 's/^UseNetworkHeuristic=.*/UseNetworkHeuristic=false/' /etc/PackageKit/PackageKit.conf

%triggerin -- xorg-x11-drv-vmmouse
# Remove unnecessary udev rules that causes problems in dom0 (#605)
rm -f /lib/udev/rules.d/69-xorg-vmmouse.rules

%triggerin -- grub2-tools
chmod -x /etc/grub.d/10_linux

%files
/etc/qubes-rpc/policy/qubes.SyncAppMenus
# Dom0 updates
/etc/cron.daily/qubes-dom0-updates.cron
/etc/yum.real.repos.d/qubes-cached.repo
/usr/bin/qubes-dom0-update
/usr/lib/qubes/patch-dnf-yum-config
%attr(4750,root,qubes) /usr/libexec/qubes/qfile-dom0-unpacker
/usr/libexec/qubes/qubes-receive-updates
/etc/qubes-rpc/qubes.ReceiveUpdates
%attr(0664,root,qubes) %config(noreplace) /etc/qubes-rpc/policy/qubes.ReceiveUpdates
%attr(0770,root,qubes) %dir /var/lib/qubes/updates
# Dracut module
/etc/dracut.conf.d/*
%dir %{_dracutmoddir}/90qubes-pciback
%{_dracutmoddir}/90qubes-pciback/*
%dir %{_dracutmoddir}/90extra-modules
%{_dracutmoddir}/90extra-modules/*
# Qrexec
/usr/sbin/qrexec-daemon
/usr/bin/qrexec-client
/usr/lib/qubes/qubes-rpc-multiplexer
# compat symlinks
/usr/lib/qubes/qrexec-client
/usr/lib/qubes/qrexec-daemon
# file copy
/usr/bin/qvm-copy-to-vm
/usr/bin/qvm-move-to-vm
/usr/bin/qvm-copy
/usr/bin/qvm-move
/usr/lib/qubes/qfile-dom0-agent
# pm-utils
/usr/lib64/pm-utils/sleep.d/52qubes-pause-vms
/usr/lib/systemd/system/qubes-suspend.service
# Others
/etc/sysconfig/modules/qubes-dom0.modules
/etc/sysconfig/modules/cpufreq-xen.modules
/etc/sudoers.d/qubes
/etc/polkit-1/rules.d/00-qubes-allow-all.rules
/etc/security/limits.d/99-qubes.conf
%config /etc/udev/rules.d/00-qubes-ignore-devices.rules
%config /etc/udev/rules.d/12-qubes-ignore-lvm-devices.rules
%attr(0644,root,root) /etc/cron.d/qubes-sync-clock.cron
/etc/cron.daily/lvm-cleanup
%config(noreplace) /etc/profile.d/zz-disable-lesspipe.sh
%config(noreplace) /etc/dnf/protected.d/qubes-core-dom0.conf
/usr/lib/systemd/system-preset/75-qubes-dom0.preset
/usr/lib/systemd/system-preset/99-qubes-default-disable.preset
/var/lib/qubes/.qubes-exclude-block-devices
# Man
%{_mandir}/man1/qvm-*.1*
%{_mandir}/man1/qubes-*.1*

%files kernel-install
/usr/lib/kernel/install.d/80-grub2.install
/usr/lib/kernel/install.d/90-xen-efi.install
/usr/lib/kernel/install.d/99-remove-bls.install

%changelog
* Thu Oct 28 2021 Qubes OS Team <qubes-devel@googlegroups.com>
- For complete changelog see: https://github.com/QubesOS/qubes-core-admin-linux.git

* Thu Oct 28 2021 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - d4b9bdc
- version 4.0.31

* Thu Oct 28 2021 Demi Marie Obenour <athena@invisiblethingslab.com> - c1f2122
- Remove old RPM database

* Thu Oct 28 2021 Demi Marie Obenour <athena@invisiblethingslab.com> - dfeafae
- Pass GPG keys to UpdateVM

* Thu Oct 28 2021 Demi Marie Obenour <athena@invisiblethingslab.com> - fe31443
- Make -e happy

* Thu Oct 28 2021 Demi Marie Obenour <athena@invisiblethingslab.com> - d862e6d
- Avoid displaying UpdateVM stderr in dom0

* Thu Oct 28 2021 Demi Marie Obenour <athena@invisiblethingslab.com> - 0a544ce
- Allow removing TemplateVM packages

* Thu Oct 28 2021 Demi Marie Obenour <athena@invisiblethingslab.com> - 6365c76
- Docuemnt that --enablerepo foo is not supported

* Tue May 04 2021 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 7c3a572
- version 4.0.30

* Tue May 04 2021 Demi Marie Obenour <demi@invisiblethingslab.com> - efe2c6f
- rpmcanon integration

* Tue May 04 2021 Demi Marie Obenour <demi@invisiblethingslab.com> - d1be4d5
- qubes-dom0-update: improved signature checking

* Thu Mar 18 2021 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - a2e4efa
- version 4.0.29

* Thu Mar 18 2021 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 3b0c641
- dom0-updates: enforce full signatures+digests package verify mode

* Thu Mar 18 2021 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 837f2b5
- dom0-updates: drop support for rpm < 4.14

* Tue Jan 12 2021 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - d7b72e0
- version 4.0.28

* Tue Jan 12 2021 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 0f79c00
- kernel-install: generate initramfs before switching default kernel

* Tue Jan 12 2021 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 772a37c
- Add .gitlab-ci.yml

* Tue Jan 12 2021 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - b0e3725
- dom0-updates: use 'incoming_dir' for getting freespace

* Tue Jan 12 2021 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 26e9a67
- Fix from Marek's comments

* Tue Jan 12 2021 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - bd3de7e
- dom0-updates: double default max updates bytes

* Tue Jan 12 2021 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 8ef229a
- dracut: install cirrus and bochs_drm into initramfs on relevant machines

* Wed Jun 24 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 4b35f85
- version 4.0.27

* Wed Jun 24 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 89f0062
- qubes-dom0-update: fix missing '&&'

* Sun Jun 07 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 9a5c3a7
- version 4.0.26

* Sun Jun 07 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 7bcdbfe
- Prevent failing if cannot set 'update-available' feature

* Sun Jun 07 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - 189ae1c
- Make ShellCheck happy and fix constant expression

* Sun Jun 07 2020 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - c89c033
- Fix typo in qubes-receive-updates

* Sat May 09 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 8cb4058
- version 4.0.25

* Sat May 09 2020 Paweł Marczewski <pawel@invisiblethingslab.com> - 10149da
- daemon: prevent collisions between port numbers

* Fri Mar 20 2020 hal18000 <hal18000@protonmail.com> - 4276746
- Fix kernel install script clobbering kernel params ending in default=

* Sun Mar 15 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - a7985cb
- version 4.0.24

* Sun Mar 15 2020 Paweł Marczewski <pawel@invisiblethingslab.com> - 9df920c
- qubes-dom0-update: don't use 'script' if not in interactive mode

* Sun Mar 15 2020 Paweł Marczewski <pawel@invisiblethingslab.com> - cbdeddf
- qubes-dom0-update: support 'list' and 'search' actions

* Sun Feb 23 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 3f0d38c
- version 4.0.23

* Fri Feb 21 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 245f8a5
- dom0-update: support rpm -K output of rpm 4.14

* Fri Jan 31 2020 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - e71ddc3
- version 4.0.22

* Wed Jan 29 2020 3hhh <tripleh@hackingthe.net> - f22e19b
- Proper argument escaping for special characters.

* Wed Jan 29 2020 3hhh <tripleh@hackingthe.net> - dcd14a4
- Don't pass stdin to VMs unless necessary.

* Tue Jan 21 2020 Pawel Marczewski <pawel@invisiblethingslab.com> - 4f2a86d
- qrexec-client: ignore SIGPIPE

* Wed Oct 23 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - cb782bd
- travis: drop R4.1 in release4.0 branch

* Wed Oct 23 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 1cf1179
- version 4.0.21

* Wed Oct 23 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 154031d
- qubes-dom0-update: fix removing backup template after the operation

* Sun Oct 20 2019 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - cc60e99
- travis: switch to bionic

* Wed Oct 09 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 3ea1fa7
- version 4.0.20

* Wed Oct 09 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 51d74f7
- Fix various issues with qubes-dom0-update

* Tue Sep 10 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 4f0152b
- version 4.0.19

* Tue Sep 10 2019 M. Vefa Bicakci <m.v.b@runbox.com> - 13d6c90
- qubes-dom0-update: Quote arguments

* Tue Sep 10 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 1b9d4f5
- Cleanup lvm archived metadata files

* Sat Mar 09 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - b6e3f36
- version 4.0.18

* Sat Mar 02 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - edf406c
- travis: fix R4.1 line

* Sat Mar 02 2019 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 1ffa0d6
- rpm: add BR: gcc

